'''Tools for making the builds.'''
from os import walk, chdir, getcwd
from os.path import join, exists, dirname, getmtime, sep
from subprocess import PIPE, run
from time import strftime
from pathlib import Path
from hashlib import md5


# TODO refactor: make class BuilderTools


def exec_cmd(cmd, check=True):
    '''Synchronously executes a command and returns its output.'''
    # ret = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True).communicate()
    # return ret[0].decode('utf-8').strip()
    proc = run(cmd, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True, check=check)
    return proc.stdout.strip() + proc.stderr.strip()


def _branch():
    '''Returns the current branch.'''
    git_branch = exec_cmd('git symbolic-ref HEAD').split('/')[-1].strip()
    print(f'git_branch result: {git_branch}')
    branches = ['master', 'rc', 'stable']
    if git_branch in branches:
        print(f'git_branch: {git_branch}')
        return git_branch
    root = str(Path(dirname(dirname(__file__))).parent) + '/'
    if 'itch' in __file__.split(sep):
        root = str(Path(dirname(__file__))) + '/'
    if __file__ == '/app/bin/pmachines':  # flatpak
        root = '/app/bin/'
    for __branch in branches:
        try:
            print(f'try: {root}' + 'assets/bld_version.txt')
            with open(root + 'assets/bld_version.txt', encoding='utf8') as fver:
                __ver = fver.read()
                print(f'ver: {__ver}')
            # if __branch in __ver:
            b2c = {'master': 'a', 'rc': 'r', 'stable': '.'}
            if __ver[1] == b2c[__branch]:
                return __branch
        except FileNotFoundError:
            print(f'file not found: {root}' + 'assets/bld_version.txt')
    return ''


def _commit():
    '''Returns the current branch.'''
    git_commit = exec_cmd('git rev-parse HEAD')[:7]
    print(f'git_commit result: {git_commit}')
    if not git_commit.startswith("Can't r"):
        return git_commit
    root = str(Path(dirname(dirname(__file__))).parent) + '/'
    if 'itch' in __file__.split(sep):
        root = str(Path(dirname(__file__))) + '/'
    if __file__ == '/app/bin/pmachines':  # flatpak
        root = '/app/bin/'
    try:
        print(f'try: {root}' + 'assets/bld_version.txt')
        with open(root + 'assets/bld_version.txt', encoding='utf8') as fver:
            __ver = fver.read()
            print(f'ver: {__ver}')
            return __ver.split('-')[1]
    except FileNotFoundError:
        print(f'file not found: {root}' + 'assets/bld_version.txt')
    return ''


def _version():
    '''Computes the version of the current build.'''
    day = strftime('%y%m%d')
    root = str(Path(dirname(dirname(__file__))).parent) + '/'
    if __file__ == '/app/bin/pmachines':  # flatpak
        root = '/app/bin/'
    if _branch() == 'stable':
        pref, _ver = '', ''
        if exists(root + 'assets/version.txt'):
            with open(root + 'assets/version.txt', encoding='utf8') as fver:
                pref = fver.read().strip() + '-'  # + _branch() + '-'
                _ver = fver.read().strip()
        ret_ver = _ver or ('0.' + day)
    else:
        # try:  we want an error!
        pref = {'master': 'a', 'rc': 'rc', '': 'runtime'}[_branch()]
        # except KeyError:
        #    pref = 'notfound'
        ret_ver = f'0{pref}{day}'
        pref = ret_ver
    bld_ver = pref + '-' + _commit()
    try:
        with open(root + 'assets/bld_version.txt', 'w', encoding='utf8') as fver:
            fver.write(bld_ver)
    except OSError:
        print("we can't write inside flatpaks, but we don't need it")
    return ret_ver


class FindFileNamesInfo:

    def __init__(self, extensions, excluding_directories=None,
                 excluding_files_ending_with=None, root_path='.'):
        self.extensions = extensions
        self.excluding_directories = excluding_directories
        self.excluding_files_ending_with = excluding_files_ending_with
        self.root_path = root_path


def find_file_names(find_info):
    #TODO avoid FindFileNamesInfo, write multiple functions with different
    # behaviors: this is a function which does more than one thing
    _extensions = find_info.extensions
    excl_dirs = find_info.excluding_directories
    excl_ends_with = find_info.excluding_files_ending_with
    root_path = find_info.root_path
    return [join(root, fname)
            for root, _, fnames in walk(root_path)
            for fname in __files_ext(fnames, _extensions)
            if not any(e_d in root.split('/') for e_d in excl_dirs or []) and
            not any(fname.endswith(e_e) for e_e in excl_ends_with or [])]


def __files_ext(fnames, _extensions):
    return [fname for fname in fnames
            if any(fname.endswith('.' + ext) for ext in _extensions)]


def __to_be_built_single(src, tgt):
    if getmtime(tgt) > getmtime(src):
        print(f'{tgt} is newer than {src}: do not build')
        return False
    with open(src, 'rb') as fsrc:
        src_content = fsrc.read()
    with open(tgt, 'rb') as ftgt:
        tgt_content = ftgt.read()
    hash_src = md5(src_content).hexdigest()
    hash_tgt = md5(tgt_content).hexdigest()
    cache = {}
    lines = []
    if exists('hash_cache.txt'):
        with open('hash_cache.txt', encoding='utf8') as fhash:
            lines = fhash.readlines()
    for line in lines:
        line_spl = line.split()
        _hash = line_spl[-1]
        fname = ' '.join(line_spl[:-1])
        cache[fname] = _hash
    if src in cache and tgt in cache:
        if hash_src == cache[src] and \
           hash_tgt == cache[tgt]:
            print(f'{tgt} and {src} are in the cache: do not build')
            return False
    print(f'{src} and {tgt} are not up-to-date: building...')
    return True


def to_be_built(tgt, srcs):
    '''Should tgt be built (i.e. is it older?) from sources srcs?'''
    if not exists(tgt):
        print(tgt + ' does not exist: building...')
        return True
    return any(__to_be_built_single(src, tgt) for src in srcs)


class InsideDir:
    '''Context manager for working inside a directory.'''

    def __init__(self, dir_):
        self.dir = dir_
        self.old_dir = getcwd()

    def __enter__(self):
        chdir(self.dir)

    def __exit__(self, exc_type, exc_val, exc_tb):
        chdir(self.old_dir)


# bld_dpath = 'build/'
# branch = _branch()
# ver = _version()
# win_fpath = '{dst_dir}{appname}-' + f'{branch}-windows.exe'
# osx_fpath = '{dst_dir}{appname}-%s-osx.zip' % branch
# flatpak_fpath = '{dst_dir}{appname}-%s-flatpak' % branch
# appimage_fpath = '{dst_dir}{appname}-' + f'{branch}-appimage'
# docs_fpath = '{dst_dir}{appname}-%s-docs.tar.gz' % branch
