import threading
from re import search


class Assert:

    @staticmethod
    def assert_render3d():
        preserve = ['camera', 'DIRECT']
        unexpected_nodes = [c for c in render.children if c.name not in preserve]
        for node in unexpected_nodes: Assert.__process_render3d_exception(node)

    @staticmethod
    def __process_render3d_exception(node):
        render.ls()
        message = f'unexpected render3d node: {node.name}'
        raise Exception(message)

    @staticmethod
    def assert_aspect2d():
        preserve = [
            'a2dBackground', 'a2dTopCenter', 'a2dTopCenterNS',
            'a2dBottomCenter', 'a2dBottomCenterNS', 'a2dLeftCenter',
            'a2dLeftCenterNS', 'a2dRightCenter', 'a2dRightCenterNS',
            'a2dTopLeft', 'a2dTopLeftNS', 'a2dTopRight',
            'a2dTopRightNS', 'a2dBottomLeft', 'a2dBottomLeftNS',
            'a2dBottomRight', 'a2dBottomRightNS', 'test_txt']
        unexpected_nodes = [c for c in aspect2d.children
                            if c.name not in preserve and not c.has_python_tag('preserve')]
        for node in unexpected_nodes: Assert.__process_aspect2d_exception(node)

    @staticmethod
    def __process_aspect2d_exception(node):
        aspect2d.ls()
        message = f'unexpected aspect2d node: {node.name}'
        raise Exception(message)

    @staticmethod
    def assert_render2d():
        preserve = ['aspect2d', 'pixel2d', 'camera2d']
        unexpected_nodes = [c for c in render2d.children if c.name not in preserve and not c.has_python_tag('preserve')]
        for node in unexpected_nodes: Assert.__process_render2d_exception(node)

    @staticmethod
    def __process_render2d_exception(self, node):
        render2d.ls()
        message = f'unexpected render2d node: {node.name}'
        raise Exception(message)

    @staticmethod
    def assert_events():
        preserve = ['window-event', 'window-closed', 'async_loader_0', 'async_loader_1',
                    'render-texture-targets-changed', 'aspectRatioChanged', 'new_scene']
        preserve_re = ['^async_loader_.*', '^click-mouse1-pg.*']
        unexpected_events = [e for e in messenger.getEvents()
                             if e not in preserve and not any(search(r, e) for r in preserve_re)]
        for e in unexpected_events: Assert.__process_event_exception(e)

    @staticmethod
    def __process_event_exception(event):
        if (acc := messenger.who_accepts(event)):
            Assert.__process_event_acceptors(acc, event)

    @staticmethod
    def __process_event_acceptors(acceptors, event):
        for key, a in acceptors.items(): Assert.__process_event_acceptor(key, a, event)

    @staticmethod
    def __process_event_acceptor(key, acceptor, event):
        #if acceptor[2]:
        #    print(f'the event {event} accepted by <{key}, {acceptor[0]}> is persistent')
        #else:
            Assert.__actually_process_event_exception(acceptor, event)

    @staticmethod
    def __actually_process_event_exception(acceptor, event):
        message = f'unexpected event: {event}, {str(acceptor)}'
        raise Exception(message)

    @staticmethod
    def assert_tasks():
        preserve = [
            'ivalLoop', 'garbageCollectStates', 'collisionLoop',
            'igLoop', 'audioLoop', 'resetPrevTransform', 'dataLoop',
            'eventManager', 'simplepbr update', 'on frame music',
            'assert_fps', 'DIRECTContextTask']
        unexpected_tasks = [t for t in taskMgr.getTasks() + taskMgr.getDoLaters()
                            if t.name not in preserve and not hasattr(t, 'preserve')]
        for t in unexpected_tasks: Assert.__process_task_exception(t)

    @staticmethod
    def __process_task_exception(task):
        message = f'unexpected task: {task.name}'
        raise Exception(message)

    @staticmethod
    def assert_buffers():
        pass
        #if RenderToTexture.buffers:
        #    raise Error()

    @staticmethod
    def assert_threads():
        thread_names = [thread.name for thread in threading.enumerate()]
        preserve = ['MainThread', 'rpc_server']
        unexpected_tasks = [t for t in thread_names if t not in preserve]
        for t in unexpected_tasks: Assert.__process_unexpected_thread(t)

    @staticmethod
    def __process_unexpected_thread(thread_name):
        message = f'unexpected thread: {thread_name}'
        raise Exception(message)
