from panda3d.core import NodePath, Point2, Point3, Texture, LVector3f, LVector2f
from ya2.utils.decorator import Decorator


class GfxTools:

    @staticmethod
    def build_model(file_path):
        m = loader.load_model(file_path)
        m = NodePathDecorator(m)  # we can't use mixins with c++ classes
        return m

    @staticmethod
    def build_empty_node(name):
        return NodePathDecorator(NodePath(name))

    @staticmethod
    def build_node_from_physics(physics_node):
        return NodePathDecorator(render.attach_new_node(physics_node))


class NodePathDecorator(NodePath, Decorator):

    def set_srgb_textures(self):
        for t in self.find_all_textures():
            f = t.get_format()
            match t.get_format():
                case Texture.F_rgba | Texture.F_rgbm:
                    f = Texture.F_srgb_alpha
                case Texture.F_rgb:
                    f = Texture.F_srgb
            t.set_format(f)

    def pos2d(self):
        p3d = base.cam.get_relative_point(self, Point3(0, 0, 0))
        p2d = Point2()
        return p2d if base.camLens.project(p3d, p2d) else None

    def pos2d_pixel(self):
        if p2d := self.pos2d():
            p = _get_pos_relative((p2d[0], 0, p2d[1]), pixel2d)
            return int(round(p[0])), int(round(-p[2]))

    def pos_as_widget(self):
        pos = self.get_pos(pixel2d)
        return int(round(pos[0])), int(round(-pos[2]))


class DirectGuiMixin():

    def pos_pixel(self):
        pos = self.get_pos(pixel2d)
        return int(round(pos[0])), int(round(-pos[2]))


class Point(Decorator):

    def screen_coord(self):
        pos3d = _get_pos_relative(LVector3f(*self), base.cam)
        pos2d = Point2()
        base.camLens.project(pos3d, pos2d)
        pos_wrt_render2d = Point3(pos2d[0], 0, pos2d[1])
        pos_wrt_aspect2d = base.aspect2d.get_relative_point(render2d, pos_wrt_render2d)
        return Point((pos_wrt_aspect2d[0], pos_wrt_aspect2d[2]))

    def from_to_points(self):
        p_from, p_to = Point3(), Point3()    # in camera coordinates
        base.camLens.extrude(LVector2f(*self), p_from, p_to)
        p_from = render.get_relative_point(base.cam, p_from)  # global coords
        p_to = render.get_relative_point(base.cam, p_to)  # global coords
        return p_from, p_to


def _get_pos_relative(pos, other):
    n = NodePath('temporary_node')
    n.set_pos(pos)
    pos = n.get_pos(other)
    n.remove_node()
    return pos
