from logging import info
from os.path import join, exists, dirname
from gettext import translation
from pathlib import Path
import sys


def is_runtime(): return not exists('main.py')


def is_appimage():
    par_path = str(Path(__file__).parent.absolute())
    is_appimage = par_path.startswith('/tmp/.mount_Pmachines')
    return is_appimage and par_path.endswith('/usr/bin')


def curr_path():
    if not is_runtime():
        return ''
    if sys.platform == 'darwin':
        return dirname(__file__) + '/../Resources/'
    # return dirname(__file__)
    par_path = str(Path(__file__).parent.absolute())
    if is_appimage():
        return str(Path(par_path).absolute())
    is_snap = par_path.startswith('/snap/')
    is_snap = is_snap and par_path.endswith('/x1')
    if is_snap:
        return str(Path(par_path).absolute())
    # return getcwd()
    curr_path = dirname(__file__)
    info('current path: %s' % curr_path)
    return curr_path + '/'


class LangMgr:

    def __init__(self, lang, domain, dpath):
        self.lang = lang
        self.domain = domain
        self.dpath = join(curr_path(), dpath)
        info('language: %s, %s' % (self.domain, self.dpath))
        self.set_lang(lang)

    @property
    def lang_codes(self):
        return [lang[1] for lang in self.eng.cfg.lang_cfg.languages]

    def set_lang(self, lang):
        self.lang = lang
        args = lang, self.domain, self.dpath
        info('setting language %s, %s, %s' % args)
        tra = translation(self.domain, self.dpath, [lang], fallback=True)
        tra.install()
